var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import axios from "axios";
import { v4 } from 'uuid';
import ExecutionEnvironment from 'exenv';
import { version } from './version';
import _ from 'lodash';
import CryptoJS from 'crypto-js';
import validator from 'validator';
import { WALLET_REGEX } from "./utils";
export class Base {
    constructor(apiKey, gameId, piiTracking = true) {
        if (!apiKey || apiKey === '') {
            throw new Error('API Key is required to initiate Helika SDK instance.');
        }
        if (!gameId || gameId === '') {
            throw new Error('Game ID is required to initiate Helika SDK instance.');
        }
        this.apiKey = apiKey;
        this.sessionID = null;
        this.gameId = gameId;
        this.sessionExpiry = new Date();
        this.baseUrl = "http://localhost:3000";
        this.piiTracking = piiTracking;
        this.enabled = true;
        this.appDetails = {
            platform_id: null,
            client_app_version: null,
            server_app_version: null,
            store_id: null,
            source_id: null,
        };
        this.anonId = this.generateAnonId();
        this.userDetails = {
            user_id: this.anonId,
            email: null,
            wallet_id: null
        };
    }
    getUserDetails() {
        return this.userDetails;
    }
    setUserDetails(details, createNewAnon = false) {
        var _a;
        if (_.isNil(details === null || details === void 0 ? void 0 : details.user_id)) {
            details = {
                user_id: this.generateAnonId(createNewAnon),
                email: undefined,
                wallet_id: undefined,
            };
        }
        if ('user_id' in details && (typeof details.user_id !== 'string' && typeof details.user_id !== 'number')) {
            throw new Error(`User Details property user_id:'${details === null || details === void 0 ? void 0 : details.user_id}' must be a string or number.`);
        }
        if ('email' in details && (details === null || details === void 0 ? void 0 : details.email) && !validator.isEmail(details === null || details === void 0 ? void 0 : details.email)) {
            throw new Error(`User Details property email:'${details === null || details === void 0 ? void 0 : details.email}' is not a valid email addess.`);
        }
        if ('email_address' in details && (details === null || details === void 0 ? void 0 : details.email_address) && !validator.isEmail(details === null || details === void 0 ? void 0 : details.email_address)) {
            throw new Error(`User Details property email_address:'${details === null || details === void 0 ? void 0 : details.email_address}' is not a valid email addess.`);
        }
        if ('wallet_id' in details && (details === null || details === void 0 ? void 0 : details.wallet_id) && !(details === null || details === void 0 ? void 0 : details.wallet_id.match(WALLET_REGEX))) {
            throw new Error(`User Details property wallet_id:'${details === null || details === void 0 ? void 0 : details.wallet_id}' is not a valid wallet addess.`);
        }
        details.user_id = (_a = details.user_id) === null || _a === void 0 ? void 0 : _a.toString();
        this.userDetails = details;
    }
    getAppDetails() {
        return this.appDetails;
    }
    setAppDetails(details) {
        this.appDetails = details;
    }
    populateDefaultValues(type, values) {
        switch (type) {
            case 'user_details': {
                let newValues = _.merge({}, values);
                newValues.email = values.email || null;
                newValues.wallet_id = values.wallet_id || null;
                return newValues;
            }
            case 'app_details': {
                let newValues = _.merge({}, values);
                newValues.platform_id = values.platform_id || null;
                newValues.client_app_version = values.client_app_version || null;
                newValues.server_app_version = values.server_app_version || null;
                newValues.store_id = values.store_id || null;
                newValues.source_id = values.source_id || null;
                return newValues;
            }
            default:
                return values;
        }
    }
    getPIITracking() {
        return this.piiTracking;
    }
    setPIITracking(piiTracking) {
        this.piiTracking = piiTracking;
        if (this.piiTracking) {
            try {
                if (ExecutionEnvironment.canUseDOM) {
                    //send Pii tracking info if it was just turned on
                    var piiEvent = this.getTemplateEvent("session_created", "session_data_updated");
                    piiEvent.event = _.merge({}, piiEvent.event, {
                        type: 'Session Data Refresh',
                        sdk_class: "Events"
                    });
                    piiEvent.event.helika_data = this.appendPIIData(this.appendHelikaData());
                    piiEvent.event.app_details = this.appDetails;
                    let event_params = {
                        id: v4(),
                        events: [piiEvent]
                    };
                    try {
                        return this.postRequest(`/game/game-event`, event_params);
                    }
                    catch (e) {
                        this.processEventSentError(e);
                    }
                }
            }
            catch (e) {
                console.error(e);
            }
        }
    }
    isEnabled() {
        return this.enabled;
    }
    setEnabled(enabled) {
        this.enabled = enabled;
    }
    getPlayerId() {
        console.warn("getPlayerId() is deprecated. Please use getUserDetails() instead.");
        return "Function is deprecated.";
    }
    setPlayerId(playerId) {
        console.warn("setPlayerId() is deprecated. Please use setUserDetails() instead.");
        // This is deprecated. No-op
    }
    setDataSettings(settings) {
        console.warn("setDataSettings() is deprecated.");
        // This is deprecated. No-op
    }
    createUAEvent(events) {
        return __awaiter(this, void 0, void 0, function* () {
            console.warn("createUAEvent() is deprecated. Please use createEvent() or createUserEvent() instead.");
            return { message: "Function is deprecated." };
        });
    }
    generateAnonId(bypassStored = false) {
        let hash = CryptoJS.SHA256(v4());
        hash = `anon_${hash.toString(CryptoJS.enc.Hex)}`;
        if (ExecutionEnvironment.canUseDOM && !bypassStored) {
            let storedHash = localStorage.getItem('helikaAnonId');
            if (!_.isEmpty(storedHash)) {
                return storedHash;
            }
            localStorage.setItem('helikaAnonId', hash);
        }
        return hash;
    }
    getTemplateEvent(event_type, event_sub_type) {
        return {
            created_at: new Date().toISOString(),
            game_id: this.gameId,
            event_type: event_type,
            event: {
                user_id: this.userDetails.user_id,
                session_id: this.sessionID,
                event_sub_type: event_sub_type ? event_sub_type : null,
            }
        };
    }
    appendHelikaData() {
        let helikaData = {
            anon_id: this.anonId,
            taxonomy_ver: 'v2',
            sdk_name: "Web",
            sdk_version: version,
            sdk_platform: 'web-sdk',
            event_source: ExecutionEnvironment.canUseDOM ? 'client' : 'server',
            pii_tracking: this.piiTracking,
        };
        return helikaData;
    }
    appendPIIData(helika_data) {
        var _a, _b, _c, _d, _e, _f;
        let piiData = {
            resolution: undefined,
            touch_support: undefined,
            device_type: undefined,
            os: undefined,
            downlink: undefined,
            effective_type: undefined,
            connection_type: undefined
        };
        if (ExecutionEnvironment.canUseDOM) {
            let connectionData = window.navigator;
            piiData = _.merge({}, piiData, {
                resolution: `${window.innerWidth}x${window.innerHeight}`,
                touch_support: (connectionData === null || connectionData === void 0 ? void 0 : connectionData.maxTouchPoints) > 0,
                device_type: ((_a = connectionData === null || connectionData === void 0 ? void 0 : connectionData.userAgentData) === null || _a === void 0 ? void 0 : _a.mobile) ? 'mobile' : (_b = connectionData === null || connectionData === void 0 ? void 0 : connectionData.userAgentData) === null || _b === void 0 ? void 0 : _b.platform,
                os: (_c = connectionData === null || connectionData === void 0 ? void 0 : connectionData.userAgentData) === null || _c === void 0 ? void 0 : _c.platform,
                downlink: (_d = connectionData === null || connectionData === void 0 ? void 0 : connectionData.connection) === null || _d === void 0 ? void 0 : _d.downlink,
                effective_type: (_e = connectionData === null || connectionData === void 0 ? void 0 : connectionData.connection) === null || _e === void 0 ? void 0 : _e.effectiveType,
                connection_type: (_f = connectionData === null || connectionData === void 0 ? void 0 : connectionData.connection) === null || _f === void 0 ? void 0 : _f.type
            });
        }
        helika_data.additional_user_info = piiData;
        return helika_data;
    }
    appendReferralData(helika_data) {
        let utms = this.refreshUtms();
        let helika_referral_link = this.refreshLinkId();
        let current_url = "";
        let referral_code = "";
        if (ExecutionEnvironment.canUseDOM) {
            current_url = window.location.href;
        }
        // todo: add referral_code info if any
        return _.merge({}, helika_data, {
            referral_data: {
                utms: utms,
                link_id: helika_referral_link,
                url: current_url,
                referral_code: referral_code
            }
        });
    }
    getUrlParam(paramName) {
        var urlParams = new URLSearchParams(window.location.search);
        return urlParams.get(paramName);
    }
    getAllUrlParams() {
        let url = window.location.href;
        if (url.indexOf('?') != -1) {
            var params = url.split('?')[1].split('&');
            return params.map(pair => {
                let values = pair.split('=');
                return {
                    key: values[0],
                    value: values[1]
                };
            });
        }
        return [];
    }
    refreshUtms() {
        try {
            if (ExecutionEnvironment.canUseDOM) {
                // Grab all utms, store it, and return it
                let utms = this.getAllUrlParams();
                if (!_.isEmpty(utms)) {
                    localStorage.setItem('helika_utms', JSON.stringify(utms));
                    return utms;
                }
                // url utms is empty
                let storedUtms = localStorage.getItem('helika_utms');
                if (storedUtms && !_.isEmpty(storedUtms)) {
                    return JSON.parse(storedUtms);
                }
            }
        }
        catch (e) {
            console.error(e);
        }
        // no utms were found 
        return null;
    }
    refreshLinkId() {
        try {
            if (ExecutionEnvironment.canUseDOM) {
                let helika_referral_link = this.getUrlParam('linkId');
                if (helika_referral_link && !_.isEmpty(helika_referral_link)) {
                    localStorage.setItem('helika_referral_link', helika_referral_link);
                    return helika_referral_link;
                }
                return localStorage.getItem('helika_referral_link');
            }
        }
        catch (e) {
            console.error(e);
        }
        return null;
    }
    getRequest(endpoint, options) {
        const url = `${this.baseUrl}${endpoint}`;
        const headers = {
            "Content-Type": "application/json",
            "x-api-key": this.apiKey,
        };
        const config = {
            params: options,
            headers: headers,
        };
        return new Promise((resolve, reject) => {
            axios
                .get(`${url}`, config)
                .then((resp) => {
                resolve(resp.data);
            })
                .catch(reject);
        });
    }
    postRequest(endpoint, options) {
        const url = `${this.baseUrl}${endpoint}`;
        const headers = {
            "Content-Type": "application/json",
            "x-api-key": this.apiKey,
        };
        const config = {
            headers,
        };
        return new Promise((resolve, reject) => {
            if (!this.enabled) {
                resolve({ message: 'Logged event' });
            }
            else {
                axios
                    .post(`${url}`, options, config)
                    .then((resp) => {
                    resolve(resp.data);
                })
                    .catch(reject);
            }
        });
    }
    sessionCreate(params) {
        return __awaiter(this, void 0, void 0, function* () {
            this.sessionID = v4();
            this.sessionExpiry = this.addMinutes(new Date(), 15);
            try {
                if (ExecutionEnvironment.canUseDOM) {
                    if (params.type === 'Session Start') {
                        let local_session_id = localStorage.getItem('sessionID');
                        let expiry = localStorage.getItem('sessionExpiry');
                        if (local_session_id && expiry && (new Date(expiry) > new Date())) {
                            this.sessionID = local_session_id;
                            localStorage.setItem('sessionExpiry', this.sessionExpiry.toString());
                            return;
                        }
                    }
                    localStorage.setItem('sessionID', this.sessionID ? this.sessionID : "");
                    localStorage.setItem('sessionExpiry', this.sessionExpiry.toString());
                }
            }
            catch (e) {
                console.error(e);
            }
            //send event to initiate session
            var initEvent = this.getTemplateEvent("session_created", "session_created");
            initEvent.event = _.merge({}, initEvent.event, {
                type: params.type,
                sdk_class: params.sdk_class
            });
            initEvent.event.helika_data = this.appendHelikaData();
            if (this.piiTracking) {
                initEvent.event.helika_data = this.appendPIIData(initEvent.event.helika_data);
            }
            initEvent.event.helika_data = this.appendReferralData(initEvent.event.helika_data);
            initEvent.event.app_details = this.appDetails;
            let event_params = {
                id: v4(),
                events: [initEvent]
            };
            try {
                return yield this.postRequest(`/game/game-event`, event_params);
            }
            catch (e) {
                this.processEventSentError(e);
            }
        });
    }
    endSession(params) {
        return __awaiter(this, void 0, void 0, function* () {
            //send event to initiate session
            var endEvent = this.getTemplateEvent("session_end", "session_end");
            endEvent.event_type = "";
            endEvent.event = _.merge({}, endEvent.event, {
                event_sub_type: 'session_end',
                sdk_class: "Events"
            });
            endEvent.event.helika_data = this.appendHelikaData();
            endEvent.event.app_details = this.appDetails;
            let event_params = {
                id: v4(),
                events: [endEvent]
            };
            try {
                return yield this.postRequest(`/game/game-event`, event_params);
            }
            catch (e) {
                this.processEventSentError(e);
            }
        });
    }
    processEventSentError(e) {
        if (e && 'response' in e && 'data' in e.response && 'message' in e.response.data &&
            e.response.data.message.startsWith('Internal server error - Invalid API key:')) {
            this.sessionID = null;
            if (ExecutionEnvironment.canUseDOM) {
                localStorage.removeItem('sessionID');
            }
            throw new Error('Error: Invalid API key. Please re-initiate the Helika SDK with a valid API Key.');
        }
        throw new Error(e.message);
    }
    addHours(date, hours) {
        date.setHours(date.getHours() + hours);
        return date.toString();
    }
    addMinutes(date, minutes) {
        date.setMinutes(date.getMinutes() + minutes);
        return date.toString();
    }
    extendSession() {
        this.sessionExpiry = this.addMinutes(new Date(), 15);
        if (ExecutionEnvironment.canUseDOM) {
            localStorage.setItem('sessionExpiry', this.sessionExpiry);
        }
        ;
    }
}
