var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { Base } from "../base";
import { EventsBaseURL } from "../index";
import ExecutionEnvironment from 'exenv';
import { v4 } from 'uuid';
import _ from "lodash";
export class EVENTS extends Base {
    constructor(apiKey, gameId, baseUrl, piiTracking = true) {
        super(apiKey, gameId, piiTracking);
        switch (baseUrl) {
            case EventsBaseURL.EVENTS_LOCAL: {
                // this.baseUrl = 'http://localhost:3000';
                this.baseUrl = "https://api-stage.helika.io/v1";
                this.enabled = false;
                break;
            }
            case EventsBaseURL.EVENTS_PROD: {
                this.baseUrl = "https://api.helika.io/v1";
                break;
            }
            case EventsBaseURL.EVENTS_DEV:
            default: {
                this.baseUrl = "https://api-stage.helika.io/v1";
                break;
            }
        }
    }
    startSession() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (ExecutionEnvironment.canUseDOM) {
                    // Todo: Move this into the Base Class once Users have been consolidated
                    return yield this.sessionCreate({
                        sdk_class: "Events",
                        type: 'Session Start'
                    });
                }
            }
            catch (e) {
                console.error(e);
            }
        });
    }
    createEvent(events) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.refreshSessionIdFromStorage();
            if (!this.sessionID) {
                throw new Error('Could not create event. No session id. Please initiate a session first (See Helika Docs).');
            }
            let params = this.prepareEventParams(events, false);
            this.extendSession();
            return this.postRequest(`/game/game-event`, params);
        });
    }
    createUserEvent(events) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            yield this.refreshSessionIdFromStorage();
            if (!this.sessionID) {
                throw new Error('Could not create event. No session id. Please initiate a session first (See Helika Docs).');
            }
            let params = this.prepareEventParams(events, true);
            let eventHasUserId = (_a = params === null || params === void 0 ? void 0 : params.events) === null || _a === void 0 ? void 0 : _a.filter((event) => {
                var _a, _b;
                return _.isNil((_b = (_a = event === null || event === void 0 ? void 0 : event.event) === null || _a === void 0 ? void 0 : _a.user_details) === null || _b === void 0 ? void 0 : _b.user_id);
            });
            if (!_.isEmpty(eventHasUserId)) {
                throw new Error('Before sending user events, user_id must be set using sdk.setUserDetails()');
            }
            this.extendSession();
            return this.postRequest(`/game/game-event`, params);
        });
    }
    refreshSessionIdFromStorage() {
        return __awaiter(this, void 0, void 0, function* () {
            if (ExecutionEnvironment.canUseDOM) {
                let local_storage_id = localStorage.getItem('sessionID');
                let expiry = localStorage.getItem('sessionExpiry');
                if (local_storage_id && expiry) {
                    if (new Date(expiry) < new Date()) {
                        yield this.refreshSession();
                    }
                    else {
                        this.sessionID = local_storage_id;
                        this.sessionExpiry = new Date(expiry);
                    }
                }
                else if (this.sessionID) { // edge case where localstorage was cleared
                    localStorage.setItem('sessionID', this.sessionID);
                }
            }
        });
    }
    refreshSession() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (ExecutionEnvironment.canUseDOM) {
                    // Todo: Move this into the Base Class once Users have been consolidated
                    return yield this.sessionCreate({
                        sdk_class: "Events",
                        type: 'Session Refresh'
                    });
                }
            }
            catch (e) {
                console.error(e);
            }
        });
    }
    prepareEventParams(events, isUserEvent) {
        let templateEvent = this.getTemplateEvent("");
        let updatedEvents = events.map((event) => {
            let givenEvent = _.merge({}, event, templateEvent);
            givenEvent.event_type = event.event_type;
            givenEvent.event.event_sub_type = event.event.event_sub_type ? event.event.event_sub_type : null;
            givenEvent.event.app_details = this.populateDefaultValues('app_details', _.merge({}, event.event.app_details, this.appDetails));
            if (isUserEvent) {
                givenEvent.event.user_details = this.populateDefaultValues('user_details', _.merge({}, event.event.user_details, this.userDetails));
            }
            givenEvent.event.helika_data = this.appendHelikaData();
            givenEvent.event.helika_data = this.appendReferralData(givenEvent.event.helika_data);
            if (!isUserEvent) {
                delete givenEvent.event.user_details;
                // replace the user_id with the 'anonId' because we just want to attach this non-user event data to a source.
                givenEvent.event.user_id = this.anonId;
            }
            return givenEvent;
        });
        var params = {
            id: v4(),
            events: updatedEvents
        };
        return params;
    }
}
